package edu.washington.cs;

import java.util.List;

import com.google.android.maps.GeoPoint;

/**
 * Reference: PhoneWand
 * class used to represent a step in the route.  Each 
 * step consists of a start location GeoPoint, an end location GeoPoint, a 
 * distance int (meters), an estimated duration int (seconds), and 
 * description String.
 */ 
public class RouteStep {
	   
    private GeoPoint startLocation;
    private GeoPoint endLocation;
    private List<GeoPoint> polyline;
    private String distance;
    private String duration;
    private String description;
        
        // Constructor
    public RouteStep(GeoPoint startLocation, GeoPoint endLocation, List<GeoPoint> polyline,
                     String distance, String duration, String description) {
            this.startLocation = startLocation;
            this.endLocation = endLocation;
            this.distance = distance;
            this.duration = duration;
            this.description = description;
            this.polyline = polyline;
    }
    
    /** Returns a GeoPoint representing the start location of this step. */
    public GeoPoint getStartLocation() {return startLocation;}
    
    /** Returns a GeoPoint representing the end location of this step. */
    public GeoPoint getEndLocation() {return endLocation;}
    
    /** Returns the polyline representing the path of this step. */
    public List<GeoPoint> getPolyline() {return polyline;}
    
    /** Returns an int representing the distance of this step (in meters). */
    public String getDistance() {return distance;}
    
    /** Returns an int representing the duration of this step (in seconds). */
    public String getDuration() {return duration;}
    
    /**
     * Returns a String which contains a textual description of this step 
     * in the route.
     */
    public String getDescription() {return description;}
    
    //author: Nara Kim
    //to flat abbreviations 
    public void setDescription(String s) {
    	description = s;
    }
    //author: Nara Kim
    //to flat abbreviations
    public void setDuration(String s) {
    	duration = s;
    }
    
    public void setDistance(String s) {
    	distance = s;
    }
    
    public boolean isEndLocationEqual(GeoPoint endLocation){
    	if(this.endLocation.getLatitudeE6() == endLocation.getLatitudeE6()
    			&& this.endLocation.getLongitudeE6() == endLocation.getLongitudeE6())
    		return true;
    	else 
    		return false;
    }
} 
