package edu.washington.cs;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import edu.washington.cs.listeners.ServerListener;

import android.os.Handler;
import android.os.Looper;
import android.util.Log;

/**
 * It handels communication to server in multi-threading manner.
 * @author Matt Moyers
 *
 */
public class ServerCommunication {
	private static final String TAG = "ServerCommunication";
	public static final int OK = 0;
	public static final int ERROR = -1;
	
	//Timeout for server connection in milliseconds
	private static final int TIMEOUT = 3000;
	//WebServer URL address
	private static final String SERVER_URL = "http://students.washington.edu/mmoyers/meetup/webserver.php?";
	
	private static Handler workerHandler;
	private static Runnable sendTask;
	private static Runnable initTask;
	
	private static ServerListener listener;
	private static String params;
	
	private static void init() {
		//Set Up Update Worker Thread
		Thread thread1 = new Thread() {
			@Override
			public void run() {
				Looper.prepare();
				
				Log.d(TAG, "Creating worker thread.");
				
				workerHandler = new Handler();
				
				Looper.loop();
			}
		};
		thread1.start();
		
		sendTask = new Runnable() { 
			public void run() {
				Log.d(TAG, "Updating info with the server.");
				
				try{
					//Check if task has been interrupted
					if(Thread.interrupted()) {
						//throw new InterruptedException();
						//TODO: do I need this.
					}
					
					//Make connection to server sending the correct information
					//TODO: change for id
					URL url = new URL(SERVER_URL + params);
					HttpURLConnection con = (HttpURLConnection) url.openConnection();
					
					con.setReadTimeout(TIMEOUT);
					con.setConnectTimeout(TIMEOUT);
					con.setRequestMethod("GET");
					con.setDoInput(true);
					con.connect();
					
					//Receive the data that the server sends us.
					BufferedReader reader = new BufferedReader(new InputStreamReader(con.getInputStream(), "UTF-8"));
					StringBuffer payload = new StringBuffer();
					while(reader.ready()) {
						payload.append(reader.readLine());
					}
					reader.close();
					Log.d(TAG, "Info recieved from the server.");
					
					//Send the data to the component
					listener.serverMessage(0, payload.toString());
					
				} catch (MalformedURLException e) {
					Log.e(TAG, "Error while attempting to form the url.");
					listener.serverMessage(-1, null);
				} catch (IOException e) {
					Log.e(TAG, "Unable to open connection to website.");
					listener.serverMessage(-1, null);
				}
			}//end run
		};//end runnable
		
		initTask = new Runnable() { 
			public void run() {
				
			}//end run
		};//end runnable
		
	}
	
	public static void sendToServer(ServerListener lis, String url) {
		listener = lis;
		params = url;
		
		workerHandler.post(sendTask);
	}
	
}
