package edu.washington.cs.core;

import edu.washington.cs.TTSManager;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.ViewGroup;
import android.widget.TextView;

/**
 * AUIActivity serves Android version of Voice Over. 
 * @author Matt Moyers
 *
 */
public abstract class AUIActivity extends Activity implements OnTouchListener, ServiceConnection {
	private static final String TAG = "AUIActivity";
	
	//TTS
	protected TTSManager tts;
	
	//Hooking UI
	//TODO: make private and fix problems bc of it
	protected ViewGroup rootLayout;
	private int[] viewLocation; //X and Y
	private static final int OVERLAY = 45;
	private static final int OVERLAY_LAYOUT = 44;
	
	//time for a touch event to become a click event
	private int clickDelay = 300;
	
	//true: to repeat same view speak in the delay
	//false: to ignore events on the same view
	private boolean isRepeatable = true;
	//delay in between repeating same button text
	private int repeatDelay = 7000;
	
	//time the last time we talked on that view
	private long lastTalkEvent;
	//the last view that was touched
	private View lastViewTouched;
	
	//last time you lifted your finger off the screen
	private long lastUpEvent;
	//delay time for a click event to become a double click event
	private int doubleTapDelay = 300;
	
	private long lastActionDown;
	
	//LOCKING BOTTOM BUTTONS
	private boolean lockBackButton = false;
	private boolean lockSearchButton = true;
	
	
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
    	//Set Up TTS
        //Log.d(TAG, "Setting up TTS.");
    	/** Tells Android to adjust the volume of music not ringer */
        setVolumeControlStream(AudioManager.STREAM_MUSIC);
        
        /** Check for Language Resource */
        //Intent checkIntent = new Intent();
    	//checkIntent.setAction(TextToSpeech.Engine.ACTION_CHECK_TTS_DATA);
    	//startActivityForResult(checkIntent, TTS_CHECK_CODE);
		
        //if(TTSManager.getInstance() == null) {
        	//Bind TTSManager
        	Intent managerIntent = new Intent(this, TTSManager.class);
        	bindService(managerIntent, this, Context.BIND_AUTO_CREATE);
        //} else {
        //	tts = TTSManager.getInstance();
        //}
    }
    
    @Override
    public void onStart() {
    	super.onStart();	
		viewLocation = new int[2];
		Log.d(TAG, "Start");
    }
    
    @Override
    public void onDestroy() {
    	super.onDestroy();
    	
    	unbindService(this);
    }

    @Override
	public boolean onTouch(View v, MotionEvent event) {
    //Check to make sure this event is valid
    	//Get X and Y position of the touch event
		float x = event.getRawX();
		float y = event.getRawY();
		CharSequence text;
		
		//Get any views at the touch coordinates
		//getComponentAt implemented in children who extends this class
		View currentViewTouched = getComponentAt(rootLayout, x, y);
		
		if(currentViewTouched == null) { 
			//the touch event does not interact with any components on screen
			if(tts != null) {
				tts.flush(TTSManager.MESSAGE_PRIORITY_LOW); //Stop any speech that might have been talking
			}
			return true; //return true to continue to get updates
		}
    	
	//Pre-Condition: This event is valid

		if((event.getActionMasked() == MotionEvent.ACTION_DOWN 					// This event is an action down
				|| event.getActionMasked() == MotionEvent.ACTION_MOVE)) {		// Or a move event ie exploring the screen
			
			if(lastViewTouched == currentViewTouched) {
				if(!isRepeatable) { //same view and we can't repeat
					return true;
				} else if((System.currentTimeMillis() - lastTalkEvent) < repeatDelay) { //same view and we can repeat but not enough time has pasted
					return true;
				}
			}
		//Pre-Condition: This event is a touching move event and its valid
			
			if((text = currentViewTouched.getContentDescription()) == null) {
				//This view has no description text
				if(currentViewTouched instanceof TextView) { 
					//if this view displays text use that to speak
					text = ((TextView) currentViewTouched).getText();
					//Log.d(TAG, "Speak " + text.toString());
					if(tts != null) {
						//tts.speak(text.toString(), TextToSpeech.QUEUE_FLUSH, null);
						tts.flush(TTSManager.MESSAGE_PRIORITY_LOW);
						tts.speak(text.toString(), TTSManager.MESSAGE_PRIORITY_LOW);
					}
				} else {
					Log.d(TAG, "Speak - Unknown TextView");
				}
			} else {
				//Log.d(TAG, "Speak " + text.toString());
				if(tts != null) {
					//tts.speak(text.toString(), TextToSpeech.QUEUE_FLUSH, null);
					tts.flush(TTSManager.MESSAGE_PRIORITY_LOW);
					tts.speak(text.toString(), TTSManager.MESSAGE_PRIORITY_LOW);
				}
			}
			
			onMove(currentViewTouched);
			
			//Update info for next touch event
			lastTalkEvent = System.currentTimeMillis();
			lastViewTouched = currentViewTouched;
				
		} else if (event.getActionMasked() == MotionEvent.ACTION_UP) {
			
			//Is this a double tap?
			if(lastViewTouched == currentViewTouched
					&& (System.currentTimeMillis() - lastUpEvent) < doubleTapDelay) {
				onClick(currentViewTouched);
				lastUpEvent = 0; //reset timer
				
			} else { //Not a double tap event
				
				lastTalkEvent = 0; //reset the talk event
				lastUpEvent = System.currentTimeMillis();
			}
			
			if(tts != null) {
				tts.flush(TTSManager.MESSAGE_PRIORITY_LOW);
			}
		}
		
		return true; //return true to continue to get updates
	}
    
	private View getComponentAt(ViewGroup group, float x, float y) {
		View object;
		
	    skip: for(int i = 0; i < group.getChildCount(); i++) {
	    	
	    	object = group.getChildAt(i);
	    	
			if(object instanceof ViewGroup) { //This object is a container
				//Look at the components in the container
				if((object = getComponentAt((ViewGroup)object, x, y)) == null) {
					continue skip;
				} else {
					return object;
				}
				
			} else {
				int xTop;
				int yTop;
				int width;
				int height;
				
				object.getLocationOnScreen(viewLocation);
				xTop =  viewLocation[0];
				yTop = viewLocation[1];
				width = object.getWidth();
				height = object.getHeight();
				
				if(y <= (yTop + height) && y >= yTop
						&& x <= (xTop + width) && x >= xTop) {
					return object;
				}
				
			}
	    	
		}
		
		return null;
	}
	
	@Override
	public void onServiceConnected(ComponentName className, IBinder service){
		Log.d(TAG, "Service Connected: " + className.getShortClassName());
		if(className.getShortClassName().equals(".TTSManager")){
			Log.d(TAG, "TTSManager Started");
			tts = ((TTSManager.ServiceBinder) service).getService();
			TTSManager.init(tts);
			ttsCreated();
		}
	}
	@Override
	public void onServiceDisconnected(ComponentName className){
		if(className.getShortClassName().equals(".TTSManager")){					
			Log.d(TAG, "TTSManager Stopped");
		}
	}
	
	public abstract void onMove(View v);
	public abstract void onClick(View v);
	public abstract void onDoubleClick(View v);
	
	public void ttsCreated() {}
}
