package edu.washington.cs.gui;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.database.Cursor;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.IBinder;
import android.provider.ContactsContract;
import android.provider.ContactsContract.CommonDataKinds;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;
import edu.washington.cs.MeetUpManager;
import edu.washington.cs.R;
import edu.washington.cs.TTSManager;
import edu.washington.cs.core.AUIActivity;

/**
 * It accesses contacts from user's phone and displays on MeetUp starting screen.
 * @author Matt Moyers
 *
 */
public class ContactActivity extends AUIActivity implements ServiceConnection {
	
	private final String TAG = "ContactActivity";
	
	private Cursor mContacts;
	
	protected final String title = "Contacts";
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(edu.washington.cs.R.layout.contacts_menu);
		
		//SET UP AUIA
		((ImageView) findViewById(R.id.interaction_overlay)).setOnTouchListener(this);
		rootLayout = (ViewGroup) findViewById(edu.washington.cs.R.id.main);
		
		//Get LIST VIEW
		ListView list = (ListView) findViewById(edu.washington.cs.R.id.contacts_list_view);
		
		// Get all contacts, ordered by name
		String[] contacts_data = new String[] { ContactsContract.Contacts._ID, ContactsContract.Contacts.DISPLAY_NAME };
		mContacts = managedQuery(ContactsContract.Contacts.CONTENT_URI, contacts_data, 
				ContactsContract.Contacts.IN_VISIBLE_GROUP+" = 1", null, ContactsContract.Contacts.DISPLAY_NAME);
		
		
		//Display all contacts in a ListView
		SimpleCursorAdapter mAdapter = new SimpleCursorAdapter(this,
				android.R.layout.simple_list_item_1, mContacts,
				new String[] { ContactsContract.Contacts.DISPLAY_NAME },
				new int[] { android.R.id.text1 });
		list.setAdapter(mAdapter);
		
		//Start MeetUpManager
		Intent managerIntent = new Intent(this, MeetUpManager.class);
		startService(managerIntent);
		bindService(managerIntent, this, Context.BIND_AUTO_CREATE);
		
		setVolumeControlStream(AudioManager.STREAM_MUSIC);
	}
	
	@Override
	public void onResume() {
		super.onResume();
		
		if(TTSManager.getInstance() != null) {
			TTSManager.getInstance().flush(TTSManager.MESSAGE_PRIORITY_LOW);
			TTSManager.getInstance().speak(getResources().getString(R.string.contacts_title), TTSManager.MESSAGE_PRIORITY_IMPORTANT);
		}
	}
	
	@Override
    public void ttsCreated() {
		if(!GUIState.hasVisitedContact()) {
			tts.speak(getResources().getString(R.string.contacts_directions), TTSManager.MESSAGE_PRIORITY_LOW);
			GUIState.visitContact();
		}
	}

	@Override
	public void onClick(View v) {
		if(v instanceof TextView) {
			TextView view = (TextView) v;
			String name = view.getText().toString();
			
			if(mContacts.moveToFirst()) {
				do { //Loop through all contacts to find this contact that we clicked
					if(mContacts.getString(1).equals(name)) { //Contact Found
						//Process Contact Info
						
						int selectedId = mContacts.getInt(0); //_ID Column
						String number;
						
						//Gather phone data from phone table
						Cursor phone = getContentResolver().query(
								CommonDataKinds.Phone.CONTENT_URI,
								new String[] { CommonDataKinds.Phone.NUMBER },
								ContactsContract.Data.CONTACT_ID + " = " + selectedId, null, null);

						if(phone.moveToFirst()) { //Does a number exist?
							number = phone.getString(0);
							System.out.println(number);
							number = number.replace("-", "");//removes - from the phone number string
							
							//Close Cursors
							phone.close();
							
							//Start next Activity
							Intent intent = new Intent(this, edu.washington.cs.gui.MeetUpActivity.class);
					        intent.putExtra("name", name);
					        intent.putExtra("id", number);
					        startActivity(intent);
					        
						} else {
							Log.d(TAG, "No phone number exists for that contact.");
						}
						
						break; //DO I NEED THIS?
					}
					
				} while(mContacts.moveToNext());
				
				Log.d(TAG, "Did not find the contact clicked in the data.");
			}
		}
		
	}

	@Override
	public void onDoubleClick(View v) {}

	@Override
	public void onMove(View v) {}
	
	@Override
	public void onServiceConnected(ComponentName className, IBinder service){
		super.onServiceConnected(className, service);
		if(className.getShortClassName().equals(".MeetUpManager")){
			Log.d(TAG, "MeetUpManager Started");
			MeetUpManager.init(((MeetUpManager.MeetUpManagerBinder) service).getService());
		}
	}
	@Override
	public void onServiceDisconnected(ComponentName className){
		super.onServiceDisconnected(className);
		if(className.getShortClassName().equals(".MeetUpManager")){					
			Log.d(TAG, "MeetUpManager Stopped");
		}
	}

}
