
package edu.washington.cs.gui;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import edu.washington.cs.MeetUpManager;
import edu.washington.cs.R;
import edu.washington.cs.TTSManager;
import edu.washington.cs.core.AUIActivity;

/**
 * MeetUp screen; user can call, text, cancel meetUp, or activate meetUp.
 * @author Matt Moyers, Nara Kim
 *
 */

public class MeetUpActivity extends AUIActivity {
	
	private final String TAG = "MeetUpActivity";
	
	protected final String title = "Meet Up";
	
	private String name;
	private String id;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(edu.washington.cs.R.layout.meetup_menu);
		
		((ImageView) findViewById(edu.washington.cs.R.id.interaction_overlay)).setOnTouchListener(this);
		rootLayout = (ViewGroup) findViewById(edu.washington.cs.R.id.main);

		Button b4 = (Button) findViewById(edu.washington.cs.R.id.button4);
		if(MeetUpManager.getInstance().areWeInAMeetUp(id)) { //We are already meeting up with this person
			b4.setText(edu.washington.cs.R.string.meetup_button4_2);
		}  else { //If we are not meeting up with this person show this button
			b4.setText(edu.washington.cs.R.string.meetup_button4);
		}
		
		name = (String) getIntent().getStringExtra("name");
		id = (String) getIntent().getStringExtra("id");
		if(id == null) {
			System.out.println("Problem with id");
		}
		
		if(name == null) {
			System.out.println("Problem with name");
			name = "Error";
		}

    }
    
	@Override
	public void onResume() {
		super.onResume();
		
		Button b4 = (Button) findViewById(edu.washington.cs.R.id.button4);
		if(MeetUpManager.getInstance().areWeInAMeetUp(id)) { //We are already meeting up with this person
			b4.setText(edu.washington.cs.R.string.meetup_button4_2);
		}  else { //If we are not meeting up with this person show this button
			b4.setText(edu.washington.cs.R.string.meetup_button4);
		}
		
		if(TTSManager.getInstance() != null) {
			TTSManager.getInstance().flush(TTSManager.MESSAGE_PRIORITY_LOW);
			TTSManager.getInstance().speak(getResources().getString(R.string.meetup_title), TTSManager.MESSAGE_PRIORITY_IMPORTANT);
		}
	}

    
    @Override
    public void ttsCreated() {
    	if(!GUIState.hasVisitedMeetUp()) {
			tts.speak(getResources().getString(R.string.meetup_directions), TTSManager.MESSAGE_PRIORITY_LOW);
    		GUIState.visitMeetUp();
    	}
    }

	@Override
	public void onClick(View v) {
		if(v instanceof Button) {
			if(((Button) v).equals((Button) findViewById(R.id.button1))) {
				//CALL
				//tts.speak("Calling " + id, TextToSpeech.QUEUE_FLUSH, null);
				//tts.flush(TTSManager.MESSAGE_PRIORITY_LOW);
				tts.speak("Calling " + id, TTSManager.MESSAGE_PRIORITY_NORMAL);
				
				Intent intent = new Intent(Intent.ACTION_CALL, Uri.parse("tel:"+id));
		        startActivity(intent);
		        
			} else if(((Button) v).equals((Button) findViewById(R.id.button2))) {
				//TEXT
				//tts.speak("Texting " + name, TextToSpeech.QUEUE_FLUSH, null);
				//tts.flush(TTSManager.MESSAGE_PRIORITY_LOW);
				tts.speak("Texting " + name, TTSManager.MESSAGE_PRIORITY_NORMAL);
				
				Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse("sms:"));
				startActivity(intent);
				
			} else if(((Button) v).equals((Button) findViewById(R.id.button3))) {
				//CANCEL MEETUP
				if(MeetUpManager.getInstance().areWeInAMeetUp()) {
					MeetUpManager.getInstance().cancelMeetUp();
					//tts.speak("Canceling your Meet Up.", TextToSpeech.QUEUE_FLUSH, null);
					//tts.flush(TTSManager.MESSAGE_PRIORITY_LOW);
					//tts.speak("Canceling your Meet Up.", TTSManager.MESSAGE_PRIORITY_NORMAL);
					
					Intent intent = new Intent(this, edu.washington.cs.gui.ContactActivity.class);	
					startActivity(intent);
				} else {
					//tts.speak("You are not currently meeting up with anyone.", TextToSpeech.QUEUE_FLUSH, null);
					//tts.flush(TTSManager.MESSAGE_PRIORITY_LOW);
					tts.speak("You are not currently meeting up with anyone.", TTSManager.MESSAGE_PRIORITY_NORMAL);
				}
				
			} else if(((Button) v).equals((Button) findViewById(R.id.button4))) {
				
				Button b4 = (Button) findViewById(edu.washington.cs.R.id.button4);
				
				if(MeetUpManager.getInstance().areWeInAMeetUp(id)) { //We are already meeting up with this person
					b4.setText(edu.washington.cs.R.string.meetup_button4_2);
					
					Intent intent = new Intent(this, edu.washington.cs.gui.DirectionActivity.class);			
				    startActivity(intent);
				}  else { //If we are not meeting up with this person show this button
					//MEETUP
					b4.setText(edu.washington.cs.R.string.meetup_button4);
					
					//tts.speak("Starting MeetUp with " + name, TextToSpeech.QUEUE_FLUSH, null);
					//tts.flush(TTSManager.MESSAGE_PRIORITY_LOW);
					//tts.speak("Starting MeetUp with " + name, TTSManager.MESSAGE_PRIORITY_NORMAL);
					
					MeetUpManager.getInstance().startMeetUp(name, id);
					
					Intent intent = new Intent(this, edu.washington.cs.gui.DirectionActivity.class);			
				    startActivity(intent);
				}
			} else {
				Log.e(TAG, "ERROR onClick Unknown View clicked");
			}
		
		}
		
	}

	@Override
	public void onDoubleClick(View v) {}

	@Override
	public void onMove(View v) {
		if(v instanceof Button) {
			if(((Button) v).equals((Button) findViewById(R.id.button4))) {
				tts.flush(TTSManager.MESSAGE_PRIORITY_LOW);
				if(MeetUpManager.getInstance().areWeInAMeetUp(id)) { //We are already meeting up with this person
					//DIRECTION
					tts.speak("Go to direction screen.", TTSManager.MESSAGE_PRIORITY_LOW);
				} else { //If we are not meeting up with this person show this button
					//MEETUP
					tts.speak("Meet Up with " + name, TTSManager.MESSAGE_PRIORITY_LOW);
				}
			}
		
		}
		
	}

}
