package edu.washington.cs.gui;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import edu.washington.cs.MeetUpManager;
import edu.washington.cs.R;
import edu.washington.cs.TTSManager;
import edu.washington.cs.core.AUIActivity;

/**
 * Travel Mode screen; user can choose travel mode from walking, driving, and transit. 
 * Selecting travel mode sends a request to server to get a route update.
 * @author Nara Kim
 *
 */
public class TravelModeActivity extends AUIActivity{
	
	private final String TAG = "TransitTypeActivity";
	
	protected final String title = "Travel Mode";
	
	protected static int TRAVEL_DRIVING = 500;
	protected static int TRAVEL_WALKING = 501;
	protected static int TRAVEL_BUS = 502;
	
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	    setContentView(edu.washington.cs.R.layout.travelmode_menu);
		
		((ImageView) findViewById(R.id.interaction_overlay)).setOnTouchListener(this);
		rootLayout = (ViewGroup) findViewById(edu.washington.cs.R.id.main);
    }
    
	@Override
	public void onResume() {
		super.onResume();
		
		if(TTSManager.getInstance() != null) {
			TTSManager.getInstance().flush(TTSManager.MESSAGE_PRIORITY_LOW);
			TTSManager.getInstance().speak(getResources().getString(R.string.travel_title), TTSManager.MESSAGE_PRIORITY_IMPORTANT);
		}
	}
	
    @Override
    public void ttsCreated() {
    	if(!GUIState.hasVisitedTravelMode()) {
    		tts.flush(TTSManager.MESSAGE_PRIORITY_LOW);
			tts.speak(getResources().getString(R.string.travel_directions), TTSManager.MESSAGE_PRIORITY_LOW);
    		GUIState.visitTravelMode();
    	}
    }
	
	@Override
	public void onClick(View v) {
		if(v instanceof Button) {
			String notChanged = "travel mode is not changed";
			if(((Button) v).equals((Button) findViewById(R.id.button1))) {
				//BY WALKING
				if(MeetUpManager.getInstance().setTravelMode("walking")){
					tts.speak("travel mode is set to walking", TTSManager.MESSAGE_PRIORITY_NORMAL);
					
					Intent in = new Intent();
					in.putExtra("mode", TRAVEL_WALKING);
					setResult(DirectionActivity.TRAVEL_CODE, in);
					finish();
				}
				else {
					tts.speak(notChanged, TTSManager.MESSAGE_PRIORITY_NORMAL);
				}
			}
			else if(((Button) v).equals((Button) findViewById(R.id.button2))) {
				//BY DRIVING
				if(MeetUpManager.getInstance().setTravelMode("driving")){
					tts.speak("travel mode is set to driving", TTSManager.MESSAGE_PRIORITY_NORMAL);
					
					Intent in = new Intent();
					in.putExtra("mode", TRAVEL_WALKING);
					setResult(DirectionActivity.TRAVEL_CODE, in);
					finish();
				}
				else {
					tts.speak(notChanged, TTSManager.MESSAGE_PRIORITY_NORMAL);
				}
			}
			else if(((Button) v).equals((Button) findViewById(R.id.button3))) {				
				//BY BUS 				
				tts.speak("Google map does not provide transit mode yet. Please select other option. " + notChanged, TTSManager.MESSAGE_PRIORITY_NORMAL);
			}
			else if(((Button) v).equals((Button) findViewById(R.id.button4))) {
				/*do nothing*/
			} 			
			else {
				Log.e(TAG, "ERROR onClick Unknown View clicked");
			}
		}	
	}
	
	@Override
	public void onDoubleClick(View v) {}
	
	@Override
	public void onMove(View v) {}
}
