package edu.washington.cs.services;

import edu.washington.cs.listeners.CompassListener;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Binder;
import android.os.IBinder;
import android.util.Log;
import android.os.Vibrator;

/**
 * A service for compass; compute cardinal direction based on where phone is heading. 
 * @author Nara Kim
 *
 */
public class CompassService extends Service implements SensorEventListener{

	public static Vibrator vibrator;
	private SensorManager sensorManager;
	private Sensor accelermeter, field;
	private float[] gravity;
	private float[] magnetic;
	private final String TAG = "CompassService";
	private final IBinder binder = new CompassBinder();
	private float degrees;
	private String direction="";
	private CompassListener compassListener;

	@Override
	public void onCreate(){	
		Log.d(TAG, "onCreate");
		super.onCreate();
		sensorManager = (SensorManager)getSystemService(SENSOR_SERVICE);
		accelermeter = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
		field = sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);	
		sensorManager.registerListener(this, accelermeter, SensorManager.SENSOR_DELAY_UI);
		sensorManager.registerListener(this, field, SensorManager.SENSOR_DELAY_UI);
		vibrator = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);	
	}

	
	@Override
	public void onDestroy(){
		Log.d(TAG, "onDestroy");
		sensorManager.unregisterListener(this);
		vibrator.cancel();
	}
	
	private void updateDirection(){
		//Log.d(TAG, "updateDirection: direction is updated.");
		float[] temp = new float[9];
		float[] R = new float[9];
		//Load rotation matrix into R
		if(gravity==null)
			Log.e(TAG, "GRAVITY iS NULL");
		SensorManager.getRotationMatrix(temp, null, gravity, magnetic);
		//Map to camera's point of view
		SensorManager.remapCoordinateSystem(temp, SensorManager.AXIS_X, SensorManager.AXIS_Z, R);
		//return the orientation values
		float[] values = new float[3];
		SensorManager.getOrientation(R, values);
		//convert to degrees
		for(int i=0; i < values.length; i++){
			Double degrees = (values[i]*180) / Math.PI;
			values[i] = degrees.floatValue();
		}
		//global variable degrees
		degrees = values[0];
		//synchronized(direction) {		
		direction =  getDirectionFromDegrees();
		//Log.d(TAG, "COMPASS: " +direction);
		if(direction != null && compassListener!=null){
			compassListener.updateCompass(direction);
			//this.stopSelf();
		}	
	}
	
	private String getDirectionFromDegrees(){
		float temp=degrees*100;
		if(temp >= -2250 && temp > 2250)
			return "n";
		else if(temp >= 2250 && temp < 6750)
			return "ne";
		else if(temp >= 6750 && temp < 11250)
			return "e";
		else if(temp >= 11250 && temp < 15750)
			return "se";
		else if(temp >= 15750 || temp < -15750)
			return "s";
		else if(temp >= -15750 && temp < -11250)
			return "sw";
		else if(temp >= -11250 && temp < -6750)
			return "w";
		else if(temp >= -6750 && temp < -2250)
			return "nw";
		else
			return null;		
	}

	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {}

	@Override
	public void onSensorChanged(SensorEvent event) {		
		switch(event.sensor.getType()){
			case Sensor.TYPE_ACCELEROMETER:				
				gravity = event.values.clone();			
				break;
			
			case Sensor.TYPE_MAGNETIC_FIELD:
				magnetic = event.values.clone();
				break;
			default:
				return;
		}
		if(gravity != null && magnetic != null)
			updateDirection();	
	}

	@Override
	public IBinder onBind(Intent intent) {
		return binder;
	}

	/**
	 * Inner class to bind CompassService to main activity.
	 * @author Nara
	 *
	 */
	public class CompassBinder extends Binder {
		public CompassService getService(){
			return CompassService.this;
		}
	}

	public void addListener(CompassListener directionActivity) {
		compassListener=directionActivity;		
	}
}
